require 'spec_helper'

describe 'performing a payment' do
  let(:prepare_payment_response) { load_response(:successful_payment_prepare) }
  let(:comfirm_payment_response) { load_response(:successful_payment_confirmaton) }

  let(:api_uri) { [Skrill::Payment::Request::API_HOST, Skrill::Payment::Request::API_PATH].join }

  let(:payment_preparation_data) do
    {
      action:     'prepare',
      email:      'bacon@email.com',
      password:   'secretbacon',
      currency:   'USD',
      subject:    'You will get paid!',
      note:       'a note!',
      bnf_email:  'customer@example.com',
      amount:     '10.0',
      frn_trn_id: '1234'
    }
  end

  let(:payment_finalize_data) do
    {
      action: 'transfer',
      sid:    'f1c08339b7c6b2084c8c83183045b0a1'
    }
  end

  before do
    stub_request(:post, api_uri).with(body: payment_preparation_data).to_return(body: prepare_payment_response)
    stub_request(:post, api_uri).with(body: payment_finalize_data).to_return(body: comfirm_payment_response)

    Skrill::Payment::Configuration.configure do |config|
      config.merchant_email    = 'bacon@email.com'
      config.merchant_password = 'secretbacon'
      config.currency          = 'USD'
      config.subject           = 'You will get paid!'
      config.note              = 'a note!'
    end
  end

  context 'Providing details via a block' do
    let(:payment) do
      Skrill::Payment.new do |payee|
        payee.email      = 'customer@example.com'
        payee.amount     = 10.00
        payee.identifier = 1234
      end
    end

    context 'when the payment was successful' do
      it 'sends the correct data to Skrill and is successful' do
        payment.deliver
        expect(payment).to be_successful
      end

      it 'has no error' do
        payment.deliver
        expect(payment.error_message).to be_nil
      end
    end

    context 'when the prepare payment was not successful' do
      let(:prepare_payment_response) { load_response(:unsuccessful_payment_prepare) }

      it 'it is not successful' do
        payment.deliver
        expect(payment).to_not be_successful
      end

      it 'has an error' do
        payment.deliver
        expect(payment.error_message).to_not be_nil
      end
    end

    context 'when the comfirm payment was not successful' do
      let(:comfirm_payment_response) { load_response(:unsuccessful_payment_confirmaton) }

      it 'it is not successful' do
        payment.deliver
        expect(payment).to_not be_successful
      end

      it 'has an error' do
        payment.deliver
        expect(payment.error_message).to_not be_nil
      end
    end
  end

  context 'Providing details via a Hash' do
    let(:payment) { Skrill::Payment.new(email: 'customer@example.com', amount: 10.00, identifier: 1234) }

    context 'when the payment was successful' do
      it 'sends the correct data to skrill and is successful' do
        payment.deliver
        expect(payment).to be_successful
      end

      it 'has no error' do
        payment.deliver
        expect(payment.error_message).to be_nil
      end
    end
  end
end
