require 'spec_helper'

describe Skrill::Payment::Request do
  describe '.post' do
    let(:api_uri) { [described_class::API_HOST, described_class::API_PATH].join }
    let(:config_data) { {} }

    after do
      WebMock.reset!
    end

    context 'when sending a valid payment' do
      before do
        stub_request(:post, api_uri).with(body: {action: 'prepare', bacon: 'such taste'}).to_return(body: load_response(:successful_payment_prepare))
        stub_request(:post, api_uri).with(body: {action: 'transfer', sid: 'f1c08339b7c6b2084c8c83183045b0a1'}).to_return(body: load_response(:successful_payment_confirmaton))
      end

      it 'returns the payment trasaction status id' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste').status_id).to eq(2)
      end

      it 'returns the payment trasaction status message' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste').status_message).to eq('processed')
      end

      it 'returns the payment trasaction id' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste').transation_id).to eq(1136226461)
      end

      it 'has no error message' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste').error_message).to eq(nil)
      end

      it 'prepares the payment' do
        Skrill::Payment::Request.post(config_data, bacon: 'such taste')

        expect(WebMock).to have_requested(:post, api_uri).with(body: {action: 'prepare', bacon: 'such taste'}).once
      end

      it 'confirms the payment' do
        Skrill::Payment::Request.post(config_data, bacon: 'such taste')

        expect(WebMock).to have_requested(:post, api_uri).with(body: {action: 'transfer', sid: 'f1c08339b7c6b2084c8c83183045b0a1'}).once
      end

      it 'is successful' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste')).to be_successful
      end

      context 'when the given option names are used' do
        let(:config_data) { {merchant_email: 'merchant@example.com'} }
        let(:reqest_data) { {email: 'customer@example.com', identifier: '123'} }

        before do
          stub_request(:post, api_uri).with(body: {action: 'prepare', email: 'merchant@example.com', bnf_email: 'customer@example.com', frn_trn_id: '123'}).to_return(body: load_response(:successful_payment_prepare))
        end

        it 'maps the given option name to the API options' do
          Skrill::Payment::Request.post(config_data, reqest_data)

          expect(WebMock).to have_requested(:post, api_uri).with(body: {action: 'prepare', email: 'merchant@example.com', bnf_email: 'customer@example.com', frn_trn_id: '123'}).once
        end
      end

      context 'when config options are set' do
        let(:config_data) { {secret: 'such chunky'} }

        before do
          stub_request(:post, api_uri).with(body: {action: 'prepare', bacon: 'such taste', secret: 'such chunky'}).to_return(body: load_response(:successful_payment_prepare))
          stub_request(:post, api_uri).with(body: {action: 'prepare', secret: 'foul chunky'}).to_return(body: load_response(:successful_payment_prepare))
        end

        it 'includes the config options' do
          Skrill::Payment::Request.post(config_data, bacon: 'such taste')

          expect(WebMock).to have_requested(:post, api_uri).with(body: {action: 'prepare', bacon: 'such taste', secret: 'such chunky'}).once
        end

        it 'overrides the config options' do
          Skrill::Payment::Request.post(config_data, secret: 'foul chunky')

          expect(WebMock).to have_requested(:post, api_uri).with(body: {action: 'prepare', secret: 'foul chunky'}).once
        end
      end
    end

    context 'when the prepare request is invalid' do
      before do
        stub_request(:post, api_uri).with(body: {action: 'prepare', bacon: 'such taste'}).to_return(body: load_response(:unsuccessful_payment_prepare))
      end

      it 'has an error message' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste').error_message).to eq('MISSING_AMOUNT')
      end

      it 'is not successful' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste')).to_not be_successful
      end
    end

    context 'when the confirm request is invalid' do
      before do
        stub_request(:post, api_uri).with(body: {action: 'prepare', bacon: 'such taste'}).to_return(body: load_response(:successful_payment_prepare))
        stub_request(:post, api_uri).with(body: {action: 'transfer', sid: 'f1c08339b7c6b2084c8c83183045b0a1'}).to_return(body: load_response(:unsuccessful_payment_confirmaton))
      end

      it 'has an error message' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste').error_message).to eq('SESSION_EXPIRED')
      end

      it 'is not successful' do
        expect(Skrill::Payment::Request.post(config_data, bacon: 'such taste')).to_not be_successful
      end
    end
  end
end
